;********************************************************************\
;
;  Name:         mscbuser.asm
;  Created by:   Stefan Ritt
;
;  Contents:     Application specific (user) part of
;                Midas Slow Control Bus protocol
;
;  $Log: mscbuser.asm,v $
;  Revision 1.1  2001/07/20 07:30:47  midas
;  Initial revision
;
;
;********************************************************************/

$INCLUDE(mscbmain.asm)
$INCLUDE(mscbutil.asm)

NODE_NAME:
        DB      'Generic node',0; maximal 16 bytes long, zero terminated

; ---- Define channels and configuration parameters returned to -----
;      the CMD_GET_INFO command

N_ADC           EQU     3       ; number of ADC channels

; ---- Channel/parameter description --------------------------------
;
; channel/param width, 
; physical units (not yet used),
; status (not yet used)
; flags (not yet used)
; 8-byte name (blank padded to form 8 bytes)

CHANNEL_INFO:
        DB      SIZE_8BIT ,0,0,0,'P2      '     ; 0
        DB      SIZE_16BIT,0,0,0,'DAC0    '     ; 1
        DB      SIZE_16BIT,0,0,0,'DAC1    '     ; 2
        DB      SIZE_16BIT,0,0,0,'ADC0    '     ; 3
        DB      SIZE_16BIT,0,0,0,'ADC1    '     ; 4
        DB      SIZE_16BIT,0,0,0,'Temp.   '     ; 5
        DB      0

CONF_PARAM:
        DB      SIZE_8BIT, 0,0,0,'ADCAvrg '
        DB      SIZE_16BIT,0,0,0,'ADCOfs  '
        DB      SIZE_16BIT,0,0,0,'ADCGain '
        DB      0

; ---- Define data space for variables -------------------------------

DSEG

; Reserve buffer space for ADC channels
READ_BUFFER:    DS      N_ADC*2

; Reserve buffer space for configuration parameters
CP_AVERAGE:     DS      1

;====================================================================
;
; Application specific init and inout/output routines
;
;====================================================================

; ---- User init function --------------------------------------------

CSEG

USER_INIT:

IF(CPU_ADUC812)
        MOV     ADCCON1,#07Ch   ; power up ADC, 14.5us conv+acq time
        MOV     ADCCON2,#0      ; select channel to convert
        MOV     DACCON,#1FH     ; power on DACs
ENDIF

IF(CPU_C8051F000)
        MOV     AMX0CF,#0       ; select single ended analog inputs
        MOV     ADC0CF,#11100000b ; 16 system clocks, gain 1
        MOV     ADC0CN,#10000000b ; enable ADC 

        MOV     REF0CN,#003h    ; enable internal reference
        MOV     DAC0CN,#080h    ; enable DAC0
        MOV     DAC1CN,#080h    ; enable DAC1
ENDIF

        RET

; ---- User write functions ------------------------------------------

; write A=chn, @R0=data

WRITE:                          ; channel 0: P2
        JNZ     W_NO_CHN0
        MOV     P0,@R0          
        RET
W_NO_CHN0:                      ; channel 1: DAC0
        CJNE    A,#1,W_NO_CHN1
        MOV     A,@R0
        INC     R0
IF(CPU_ADUC812)
        MOV     DAC0H,@R0
        MOV     DAC0L,A
ENDIF
IF(CPU_C8051F000)
        MOV     DAC0L,A
        MOV     DAC0H,@R0
ENDIF
        RET
W_NO_CHN1:                      ; channel 2: DAC1
        CJNE    A,#2,W_NO_CHN2
        MOV     A,@R0
        INC     R0
IF(CPU_ADUC812)
        MOV     DAC1H,@R0
        MOV     DAC1L,A
ENDIF
IF(CPU_C8051F000)
        MOV     DAC1L,A
        MOV     DAC1H,@R0
ENDIF
        RET
W_NO_CHN2:
        RET

; ---- User read functions -------------------------------------------

; read A=chn
; return: @R0=data, A=length
READ:   
        JNZ     R_NO_CHN0       ; channel 0: P2
        MOV     A,#1
        MOV     @R0,P2
        RET
R_NO_CHN0:                      ; channel 1: DAC0
        CJNE    A,#1,R_NO_CHN1
        MOV     @R0,DAC0L
        INC     R0
        MOV     @R0,DAC0H
        MOV     A,#2
        RET
R_NO_CHN1:                      ; channel 2: DAC1
        CJNE    A,#2,R_NO_CHN2
        MOV     @R0,DAC1L
        INC     R0
        MOV     @R0,DAC1H
        MOV     A,#2
        RET
R_NO_CHN2:                      ; channel 3: ADC0
        CJNE    A,#3,R_NO_CHN3
        MOV     A,#READ_BUFFER
        SJMP    R_DO
        
R_NO_CHN3:                      ; channel 4; ADC1
        CJNE    A,#4,R_NO_CHN4
        MOV     A,#READ_BUFFER+2
        SJMP    R_DO
        
R_NO_CHN4:                      ; channel 5: TEMP.
        CJNE    A,#5,R_NO_CHN5
        MOV     A,#READ_BUFFER+4

R_DO:
        MOV     R1,A            ; copy 1st byte
        MOV     A,@R1
        MOV     @R0,A
        INC     R0              ; copy 2nd byte 
        INC     R1
        MOV     A,@R1           
        MOV     @R0,A
        MOV     A,#2            ; return 2 bytes
        RET

R_NO_CHN5:
        MOV     A,#0            ; return nothing
        RET

; ---- Configuration parameter write function ------------------------

WRITE_CONF:                     ; channel 0: CP_AVERAGE
        JNZ     WCONF_CH1
        MOV     CP_AVERAGE,@R0
        RET

WCONF_CH1:                      ; channel 1: ADC offset
        CJNE    A,#1,WCONF_CH2
IF(CPU_ADUC812)
        INC     R0
        MOV     A,@R0
        CJNE    A,#0FFh,WCONF_WCH1
        RET                     ; do not write of 0xFFFF
WCONF_WCH1:        
        MOV     ADCOFSH,A
        DEC     R0
        MOV     A,@R0
        MOV     ADCOFSL,A
ENDIF
        RET
WCONF_CH2:                      ; channel 2: ADC gain
IF(CPU_ADUC812)
        INC     R0
        MOV     A,@R0
        CJNE    A,#0FFh,WCONF_WCH2
        RET                     ; do not write if 0xFFFF
WCONF_WCH2:        
        MOV     ADCGAINH,A
        DEC     R0
        MOV     A,@R0
        MOV     ADCGAINL,A
ENDIF
        RET

; ---- Configuration parameter read function -------------------------

READ_CONF:                      ; channel 0: CP_AVERAGE
        JNZ     RCONF_CH1
        MOV     @R0,CP_AVERAGE
        MOV     A,#1
        RET

RCONF_CH1:                      ; channel 1: ADC offset
        CJNE    A,#1,RCONF_CH2
IF(CPU_ADUC812)
        MOV     A,ADCOFSL
        MOV     @R0,A
        INC     R0
        MOV     A,ADCOFSH
        MOV     @R0,A
        MOV     A,#2
ENDIF
        RET
RCONF_CH2:                      ; channel 2: ADC gain
IF(CPU_ADUC812)
        MOV     A,ADCGAINL
        MOV     @R0,A
        INC     R0
        MOV     A,ADCGAINH
        MOV     @R0,A
        MOV     A,#2
ENDIF
        RET

; --------------------------------------------------------------------

; read ADC and store it in READ_BUFFER, A=chn
; modifies: A,R0,R1,R2,R3

IF(CPU_ADUC812)
ADC_CONV        EQU     SCONV
ADC_DATAL       EQU     ADCDATAL
ADC_DATAH       EQU     ADCDATAH
ADC_MUX         EQU     ADCCON2
ENDIF
IF(CPU_C8051F000)
ADC_CONV        EQU     ADBUSY
ADC_DATAL       EQU     ADC0L
ADC_DATAH       EQU     ADC0H
ADC_MUX         EQU     AMX0SL
ENDIF

READ_ADC:
        ANL     A,#00001111b    ; mask channel
        MOV     ADC_MUX,A       ; select ADC channel

        MOV     R3,CP_AVERAGE   ; average N conversions
        MOV     R1,#0           ; sum LSB
        MOV     R2,#0           ; sum MSB
ADC_LOOP:
        SETB    ADC_CONV        ; start conversion
        JB      ADC_CONV,$      ; repeat until conversion finished

        MOV     A,R1
        ADD     A,ADC_DATAL
        MOV     R1,A

        MOV     A,R2
        MOV     A,ADC_DATAH
        ANL     A,#00001111b
        ADDC    A,R2
        MOV     R2,A

        DJNZ    R3,ADC_LOOP
        
        CLR     EA              ; disable interrupts

        MOV     A,R1            ; move sum to read buffer
        MOV     @R0,A
        INC     R0
        MOV     A,R2
        MOV     @R0,A
        INC     R0

        SETB    EA              ; enable interrupts

        RET

;---- arbitrary user function, called via CMD_USER command -----------

; on call:
;   @R0=parameters, A=length
;
; on return:
;   @R1=data, A=length

USER_FUNC:
        ; output parameters to LCD
        CALL    LCD_CLEAR
        MOV     A,@R0
        CALL    LCD_HEX
        INC     R0
        MOV     A,@R0
        CALL    LCD_HEX

        ; return some dummy result
        MOV     @R1,#012h
        INC     R1
        MOV     @R1,#034h
        MOV     A,#2
        RET

;---- user loop functions called regularly ---------------------------

USER_LOOP:
        JB      FREEZE_MODE,USER_RET ; don't take data in freeze mode

        MOV     R0,#READ_BUFFER 
        
        MOV     A,#0            ; read ADC0
        CALL    READ_ADC
        
        MOV     A,#1            ; read ADC1
        CALL    READ_ADC
        
        MOV     A,#7            ; read ADC7 (LM35)
        CALL    READ_ADC
        
USER_RET:
        RET

;--------------------------------------------------------------------

END
