// -*- C++ -*-
//
// GsimDetview.h
// Drawing function
//
// Author:  Mitsuhiro YAMAGA (yamaga@post.kek.jp)
// Created: Thu Jul 12 15:12:46 JST 2001
//
// $Id$
//
// Revision History
//
// $Log$
//

#ifndef GSIMGENERAL_H_INCLUDED
#define GSIMGENERAL_H_INCLUDED

//
#include <string>
#include <time.h>
#include <sys/times.h>
#include <unistd.h>
#include "CLHEP/Vector/ThreeVector.h"
#include "TupleManager/TupleManager.h"


////////////////

//
#define  MAX_MECH  10000  // Gukine.cc


////////////////

// GsimDetView.cc
extern int    isGxsim;
extern int    disp_2buf;
extern float  disp_zoom[];
extern float  disp_center[];
extern float  disp_rotth;
extern float  disp_rotph;
extern float  disp_rotps;

// GsimGeom.cc
extern int    DetUseFBAR[];
extern int    DetUseCBAR[];
extern int    DetUseCSIC[];
extern int    DetUseCSEG[];
extern int    DetUseCC01[];
extern int    DetUseCC02[];
extern int    DetUseCC03[];
extern int    DetUseCC04[];
extern int    DetUseCC05[];
extern int    DetUseBMCT[];
extern int    DetUseCHGV[];

// GsimMain.cc
extern int        Display;
extern int        SaveMode;
extern TMNtuple*  Ituple;
extern TMNtuple*  Otuple;
extern TMNtuple*  Ogenpart;
extern TMNtuple*  Ogsimrand;

// GsimMate.cc
extern float  mfstep;

// Gukine.cc
extern float  TargetZ;
extern float  MoveGenZ[];
extern int    GenMechanism[];


////////////////

// GsimDetView.cc
void  GsimDetView( int iview );

// GsimEvent.cc
int   GsimEvent( int loop );
int   GsimEventSkip( int skip );

// GsimFFuser.cc
void  GsimFFuser();

// GsimGenParticle.cc
int   GsimGenParticle();

// GsimGeom.cc
void  GsimRotationMatrix();
void  GsimGeom();

// GsimGeom_XXX.cc
void  GsimGeom_FrontBarrel();
void  GsimGeom_CentralBarrel();
void  GsimGeom_Csi();
void  GsimGeom_CC03();

// GsimHits.cc
int   GsimHits();
int   GsimHitsCompress();

// GsimMain.cc
void  GsimUsage();
int   GsimParams( int argc, const char* argv[] );
int   GsimEventLoop();

// GsimMate.cc
void  GsimMate();

// GsimParticleGun.cc
void  GsimParticleGun( int pid, 
                       double pmin,  double pmax,
                       double pthmin,double pthmax, 
                       double pphmin,double pphmax,
                       double rmin,  double rmax,
                       double phmin, double phmax,
                       double zmin,  double zmax,
                       Hep3Vector& p, Hep3Vector& vtx );
void  GsimKlongDecayMode( int mode );
void  GsimGenKlongCol6( double z,int absorber, Hep3Vector& p,Hep3Vector& vtx );
void  GsimGenKlongTarget( double z, Hep3Vector& p, Hep3Vector& vtx );

// Uglast
void  Uglast();

// Uginit
void  Uginit();

// Ugeom
void  Ugeom();


// gsim_ffread.F
extern "C" void  gsim_ffread_();

// gsim_psfile.F
extern "C" {
  void  gsim_open_epsfile_( int*,const char*, int );
  void  gsim_open_psfile_(  int*,const char*, int );
  void  gsim_close_psfile_( int* );
}


////////////////
// inline

//
inline double
deg2rad( double deg ){ return( deg/180.*M_PI ); }

//
inline double
rad2deg( double rad ){ return( rad/M_PI*180. ); }

//
inline void
OPEN_PSFILE( int lun, std::string fname )
{
  if ( fname.find( ".eps" ) < fname.length() ){
    gsim_open_epsfile_( &lun, fname.c_str(), fname.length() );
  }
  else{
    gsim_open_psfile_( &lun, fname.c_str(), fname.length() );
  }
}

//
inline void
CLOSE_PSFILE( int lun )
{
  gsim_close_psfile_( &lun );
}


#endif // GSIMGENERAL_H_INCLUDED
