// -*- C++ -*-
//
// GsimGeom.cc
// Geometry definition function
//
// Author:  Mitsuhiro YAMAGA (yamaga@post.kek.jp)
// Created: Thu Jul 12 15:12:46 JST 2001
//
// $Id$
//
// Revision History
//
// $Log$
//

#include <iostream>
#include <string>
#include <stdlib.h>
#include <math.h>
#include "CLHEP/Vector/ThreeVector.h"
#include "ccfortran/G321Common.h"
#include "ccfortran/ccfortran.h"
#include "GsimGeneral.h"
#include "GsimGeom.h"


//////
#define  DEBUG  0


//////
// FFkey
int  DetUseFBAR[] = { 1, 1 };
int  DetUseCBAR[] = { 1, 1 };
int  DetUseCSIC[] = { 1, 1 };
int  DetUseCSEG[] = { 1, 1 };
int  DetUseCC01[] = { 1, 1 };
int  DetUseCC02[] = { 1, 1 };
int  DetUseCC03[] = { 1, 1 };
int  DetUseCC04[] = { 1, 1 };
int  DetUseCC05[] = { 1, 1 };
int  DetUseBMCT[] = { 1, 1 };
int  DetUseCHGV[] = { 1, 1 };


//////
void
GsimRotationMatrix()
{
  //
  for( int i=0;i<32;i++ ){
    GSROTM( grot_z_32+i, 90, i*360./32,  90, i*360./32+90,  0, 0 );
  }
  //
  for( int i=0;i<32;i++ ){
    GSROTM( grot_x90_z_32+i, 90, i*360./32,  0, 0,  90, -90+i*360./32 );
  }
  //
  for( int i=0;i<32;i++ ){
    GSROTM( grot_y90_z_32+i, 180, 0,  90, 90+i*360./32,  90, i*360./32 );
  }
  
}


//////
void
GsimGeom()
{
  if ( gcflag_.ISWIT[3-1] > 0 )
    std::cerr << "GsimGeom" << std::endl; 

  //
  GsimRotationMatrix();

  //
  int  ivol=0;

  // Mother volume
  float p_moth[] = {400./2, 400./2, 2400./2 }; // x,y,z
  GSVOLU( "MOTH","BOX ",n_vacuum,p_moth,sizeof(p_moth)/sizeof(float),ivol );

  // Front Barrel
  GsimGeom_FrontBarrel();

  // Central Barrel
  GsimGeom_CentralBarrel();

  // CsI crystal
  GsimGeom_Csi();

  // CC01
  double  cc01_len  = 25;
  double  cc01_rmin = 4.3;
  double  cc01_rmax = 31;
  float  p_cc01[] = { cc01_rmin,cc01_rmax, cc01_len/2 };
  GSVOLU( "CC01","TUBE",n_lead,p_cc01,sizeof(p_cc01)/sizeof(float),ivol );
  double c_cc01_z = cc01_len/2;
  if ( DetUseCC01[0] ){ //////
    GSPOS( "CC01", 1, "MOTH", 0,0,c_cc01_z, 0, "ONLY" );
  }

  // CC02
  double  fbar_len  = 275; // from GsimGeom_FrontBarrel.cc
  double  cc02_len  = 25;
  double  cc02_rmin = 6.2;
  double  cc02_rmax = 31;
  float  p_cc02[] = { cc02_rmin,cc02_rmax, cc02_len/2 };
  GSVOLU( "CC02","TUBE",n_lead,p_cc02,sizeof(p_cc02)/sizeof(float),ivol );
  double c_cc02_z = fbar_len - cc02_len/2;
  if ( DetUseCC02[0] ){ //////
    GSPOS( "CC02", 1, "MOTH", 0,0,c_cc02_z, 0, "ONLY" );
  }

  // CC03
  GsimGeom_CC03();

  // CC04
  double  cc04_len  = 40;
  double  cc04_rmin = 6.5;
  double  cc04_rmax = 60./2;
  float  p_cc04[] = { -360./4/2, 360, 4, 2,
                      -cc04_len/2,cc04_rmin,cc04_rmax,
                      +cc04_len/2,cc04_rmin,cc04_rmax  };
  GSVOLU( "CC04","PGON",n_lead,
          p_cc04,sizeof(p_cc04)/sizeof(float),ivol );
  double c_cc04_z = 700 + cc04_len/2;
  if ( DetUseCC04[0] ){ //////
    GSPOS( "CC04", 1, "MOTH", 0,0,c_cc04_z, 0, "ONLY" );
  }

  // CC05
  double  cc05_len  = 40;
  double  cc05_rmin = 7.0;
  double  cc05_rmax = 60./2;
  float  p_cc05[] = { -360./4/2, 360, 4, 2,
                      -cc05_len/2,cc05_rmin,cc05_rmax,
                      +cc05_len/2,cc05_rmin,cc05_rmax  };
  GSVOLU( "CC05","PGON",n_lead,
          p_cc05,sizeof(p_cc05)/sizeof(float),ivol );
  double c_cc05_z = 850 + cc05_len/2;
  if ( DetUseCC05[0] ){ //////
    GSPOS( "CC05", 1, "MOTH", 0,0,c_cc05_z, 0, "ONLY" );
  }

  // Beam Catcher
  double  bmct_len  = 20;
  double  bmct_rmin = 0;
  double  bmct_rmax = 15./2;
  float  p_bmct[] = { bmct_rmax, bmct_rmax, bmct_len/2 };
  GSVOLU( "BMCT","BOX ",n_scintillator,
          p_bmct,sizeof(p_bmct)/sizeof(float),ivol );
  double c_bmct_z = 895 + bmct_len/2;
  if ( DetUseBMCT[0] ){ //////
    GSPOS( "BMCT", 1, "MOTH", 0,0,c_bmct_z, 0, "ONLY" );
  }

  // Charged Veto
  // CsI size from GsimGeom_CC03
  double  csir_len  = 80;    // CsI region
  double  c_csir_z  = 610 - 5 + csir_len/2;
  // inner part
  int     chv_ndiv  = 32;
  double  chv_thick = 0.6; // 6 mm scintillator
  double  chv1_len  = 40 - chv_thick;
  double  chv1_rmin = 8;
  double  chv1_rmax = chv1_rmin + chv_thick;
  float  p_chv1[] = { -360./32/2, 360./32, 1, 2,
                      -chv1_len/2, chv1_rmin, chv1_rmax,
                      +chv1_len/2 ,chv1_rmin, chv1_rmax };
  GSVOLU( "CHV1","PGON",n_scintillator,
          p_chv1,sizeof(p_chv1)/sizeof(float),ivol );
  double c_chv1_z = c_csir_z - csir_len/2 - chv1_len/2;
  for( int i=0;i<chv_ndiv;i++ ){
    if ( DetUseCHGV[0] ){ //////
      GSPOS( "CHV1", i+1, "MOTH", 0,0,c_chv1_z, grot_z_32+i, "ONLY" );
    }
  }
  // plate
  double  chv2_len  = chv_thick;
  double  chv2_rmin = chv1_rmin;
  double  chv2_rmax = 99 - 45;
  float  p_chv2[] = { -360./32/2, 360./32, 1, 2,
                      -chv2_len/2, chv2_rmin, chv2_rmax,
                      +chv2_len/2 ,chv2_rmin, chv2_rmax };
  GSVOLU( "CHV2","PGON",n_scintillator,
          p_chv2,sizeof(p_chv2)/sizeof(float),ivol );
  double c_chv2_z = c_chv1_z - chv1_len/2 - chv2_len/2;
  for( int i=0;i<chv_ndiv;i++ ){
    if ( DetUseCHGV[0] ){ //////
      GSPOS( "CHV2", i+1, "MOTH", 0,0,c_chv2_z, grot_z_32+i, "ONLY" );
    }
  }
  // bending part
  double  chv3_rmax  = 45; // !!!
  double  chv3_rmin  = chv3_rmax - chv_thick;
  double  chv3_len   = chv2_rmax*tan(M_PI/chv_ndiv)*2 +0.0; // 0.0cm overlap
  double  chv3_phmin = 0;
  double  chv3_phmax = 90;
  Hep3Vector  chv3_z1(0,0,-1),chv3_z2(0,0,1);
  chv3_z1.rotateX(-M_PI/chv_ndiv);
  chv3_z2.rotateX( M_PI/chv_ndiv);
  float  p_chv3[] = { chv3_rmin,chv3_rmax,chv3_len/2,
                      chv3_phmin,chv3_phmax,
                      chv3_z1.x(),chv3_z1.y(),chv3_z1.z(),
                      chv3_z2.x(),chv3_z2.y(),chv3_z2.z()  };
  GSVOLU( "CHV3","CTUB",n_scintillator,
          p_chv3,sizeof(p_chv3)/sizeof(float),ivol );
  for( int i=0;i<chv_ndiv;i++ ){
    Hep3Vector  c_chv3( chv2_rmax, 0, c_chv1_z+chv1_len/2+5 );
    c_chv3.rotateZ( 2*M_PI/chv_ndiv*i );
    int  rotm = grot_y90_z_32+(24+i*(32/chv_ndiv))%32;
    if ( DetUseCHGV[0] ){ //////
      GSPOS( "CHV3",i+1,"MOTH", c_chv3.x(),c_chv3.y(),c_chv3.z(),
             rotm, "ONLY" );
    }
  }
  // outer part
  double  chv4_len  = 10;
  double  chv4_rmax = chv2_rmax + chv3_rmax;
  double  chv4_rmin = chv4_rmax - chv_thick;
  float  p_chv4[] = { -360./32/2, 360./32, 1, 2,
                      -chv4_len/2, chv4_rmin, chv4_rmax,
                      +chv4_len/2 ,chv4_rmin, chv4_rmax };
  GSVOLU( "CHV4","PGON",n_scintillator,
          p_chv4,sizeof(p_chv4)/sizeof(float),ivol );
  double c_chv4_z = c_csir_z - csir_len/2 + 5 + chv4_len/2;
  for( int i=0;i<chv_ndiv;i++ ){
    if ( DetUseCHGV[0] ){ //////
      GSPOS( "CHV4", i+1, "MOTH", 0,0,c_chv4_z, grot_z_32+i, "ONLY" );
    }
  }
  //
}


//////
extern"C"
void
gsim_geom_()
{
  GsimGeom();
}

