// -*- C++ -*-
//
// GsimGeom_CC03.cc
// Geometry definition function
//
// Author:  Mitsuhiro YAMAGA (yamaga@post.kek.jp)
// Created: Thu Jul 12 15:12:46 JST 2001
//
// $Id$
//
// Revision History
//
// $Log$
//

#include <iostream>
#include <string>
#include <stdlib.h>
#include <math.h>
#include "CLHEP/Vector/ThreeVector.h"
#include "ccfortran/G321Common.h"
#include "ccfortran/ccfortran.h"
#include "GsimGeneral.h"
#include "GsimGeom.h"


//////
#define  DEBUG  0


//////
// FFkey
extern int  DetUseCC03[];


//////
void
GsimGeom_CC03()
{
  //
  int  ivol=0;

  // CsI size from GsimGeom_Csi.cc
  double  csic_len  = 30;    // CsI crystal size 
  double  csic_xy   = 7;     // 
  double  csic_gap  = 0.015; // reflector sheet
  double  csir_len  = 80;    // CsI region
  double  c_csir_z  = 610 - 5 + csir_len/2;

  // CC03
  double  cc03_len = 40;
  double  cc03_xyo = (csic_xy+csic_gap*2)*5; // = 35.15 cm
  double  cc03_xyi = 11.2;
  float  p_cc03[] = { -360./4/2, 360, 4, 2,
                      -cc03_len/2, cc03_xyi/2, cc03_xyo/2,
                      +cc03_len/2, cc03_xyi/2, cc03_xyo/2 };
  GSVOLU( "CC03","PGON",n_vacuum,p_cc03,sizeof(p_cc03)/sizeof(float),ivol );
  double c_cc03_z = -csir_len/2 + cc03_len/2;
  if ( DetUseCC03[0] ){ //////
    GSPOS( "CC03", 1, "CSI ", 0,0,c_cc03_z, 0, "ONLY" );
  }

  // CC03 Inner Support SUS Frame
  double  cc3i_len = cc03_len;
  double  cc3i_xyo = 11.6; // = cc3m_x - cc3m_y
  double  cc3i_xyi = 11.2; // 2 mm thick x 2
  float  p_cc3i[] = { -360./4/2, 360, 4, 2,
                      -cc3i_len/2, cc3i_xyi/2, cc3i_xyo/2,
                      +cc3i_len/2, cc3i_xyi/2, cc3i_xyo/2 };
  GSVOLU( "CC3I","PGON",n_iron,p_cc3i,sizeof(p_cc3i)/sizeof(float),ivol );
  if ( DetUseCC03[0] ){ //////
    GSPOS( "CC3I", 1, "CC03", 0,0,0, 0, "ONLY" );
  }

  // CC03 module
  double  cc3w_thick = 0.1;  // tungsten
  double  cc3s_thick = 0.34; // scintillator
  double  cc3s_gap   = 0.00; // wrapping sheet of scintillator (no sheet?)
  double  cc3m_len = cc03_len;
  double  cc3m_x   = 22.94;
  double  cc3m_y   = 11.34; // 26*scinti + 25*tungsten
  float  p_cc3m[] = { cc3m_x/2, cc3m_y/2, cc3m_len/2 };
  GSVOLU( "CC3M","BOX ",n_vacuum,p_cc3m,sizeof(p_cc3m)/sizeof(float),ivol );
  double c_cc3m_x = (cc3m_x+cc3m_y)/2-cc3m_x/2;
  double c_cc3m_y = (cc3m_x+cc3m_y)/2-cc3m_y/2;
  GSPOS( "CC3M", 1, "CC03",  c_cc3m_y, c_cc3m_x,0, grot_z_32+32*3/4, "ONLY" );
  GSPOS( "CC3M", 2, "CC03", -c_cc3m_x, c_cc3m_y,0, 0,                "ONLY" );
  GSPOS( "CC3M", 3, "CC03", -c_cc3m_y,-c_cc3m_x,0, grot_z_32+32*1/4, "ONLY" );
  GSPOS( "CC3M", 4, "CC03",  c_cc3m_x,-c_cc3m_y,0, grot_z_32+32*2/4, "ONLY" );

  // CC03 components
  float  p_cc3s[] = { cc3m_x/2, cc3s_thick/2, cc3m_len/2 };
  GSVOLU( "CC*S","BOX ",n_scintillator,
          p_cc3s,sizeof(p_cc3s)/sizeof(float),ivol );
  for( int i=0;i<26;i++ ){
    double  dd = cc3w_thick + cc3s_gap*2 + cc3s_thick;
    double  cy = -cc3m_y/2 + dd*i + cc3s_gap + cc3s_thick/2;
    if ( DetUseCC03[0] ){ //////
      GSPOS( "CC*S", i+1, "CC3M", 0,cy,0, 0, "ONLY" );
    }
  }
  float  p_cc3w[] = { cc3m_x/2, cc3w_thick/2, cc3m_len/2 };
  GSVOLU( "CC*W","BOX ",n_tungsten,
          p_cc3w,sizeof(p_cc3w)/sizeof(float),ivol );
  for( int i=0;i<25;i++ ){
    double  dd = cc3w_thick + cc3s_gap*2 + cc3s_thick;
    double  cy = -cc3m_y/2 + dd*(i+1) - cc3w_thick/2;
    if ( DetUseCC03[0] ){ //////
      GSPOS( "CC*W", i+1, "CC3M", 0,cy,0, 0, "ONLY" );
    }
  }

  //
}

