// -*- C++ -*-
//
// GsimGeom_Csi.cc
// Geometry definition function
//
// Author:  Mitsuhiro YAMAGA (yamaga@post.kek.jp)
// Created: Thu Jul 12 15:12:46 JST 2001
//
// $Id$
//
// Revision History
//
// $Log$
//

#include <iostream>
#include <string>
#include <stdlib.h>
#include <math.h>
#include "CLHEP/Vector/ThreeVector.h"
#include "ccfortran/G321Common.h"
#include "ccfortran/ccfortran.h"
#include "GsimGeneral.h"
#include "GsimGeom.h"


//////
#define  DEBUG  0


//////
// FFkey
extern int  DetUseCSIC[];
extern int  DetUseCSEG[];


//////
void
GsimGeom_Csi()
{
  //
  int  ivol=0;

  // CsI crystal
  double  csic_len  = 30;    // CsI crystal size
  double  csic_xy   = 7;     // 
  double  csic_gap  = 0.015; // reflector sheet
  float  p_csic[] = { csic_xy/2, csic_xy/2, csic_len/2 };
  GSVOLU( "CSIC","BOX ",n_csi,p_csic,sizeof(p_csic)/sizeof(float),ivol );
  // deformed CsI crystals
  GSVOLU( "CSDB","BOX ",n_csi,0,0,ivol );
  GSVOLU( "CSDT","TRAP",n_csi,0,0,ivol );
  double csdt_top[7],csdt_bot[7],csdt_hi[7],csdb_y[7];
  // #1
  csdt_top[1] = 5.1228;
  csdt_bot[1] = 7;
  csdt_hi[1]  = 0.1752;
  csdb_y[1]   = 6.8248;
  // #2
  csdt_top[2] = 5.7604;
  csdt_bot[2] = 6.8098;
  csdt_hi[2]  = 7;
  // #3
  csdt_top[3] = 4.1639;
  csdt_bot[3] = 5.7604;
  csdt_hi[3]  = 7;
  // #4
  csdt_top[4] = 5.2141;
  csdt_bot[4] = 7;
  csdt_hi[4]  = 0.7560;
  csdb_y[4]   = 6.2440;
  // #5
  csdt_top[5] = 4.7346;
  csdt_bot[5] = 7;
  csdt_hi[5]  = 1.4439;
  csdb_y[5]   = 5.5561;
  // #6
  csdt_top[6] = 0.4929;
  csdt_bot[6] = 7;
  csdt_hi[6]  = 5.8158;
  csdb_y[6]   = 1.1842;

  // CsI region
  double  csir_len  = 80;    // CsI region
  double  csir_rmin = 0;
  double  csir_rmax = 97.5;
  float  p_csir[] = { csir_rmin,csir_rmax, csir_len/2 };
  GSVOLU( "CSI ","TUBE",n_vacuum,p_csir,sizeof(p_csir)/sizeof(float),ivol );
  double c_csir_x = 0;
  double c_csir_y = 0;
  double c_csir_z = 610 - 5 + csir_len/2;
  if ( DetUseCSIC[0] || DetUseCSEG[0] )
    GSPOS( "CSI ", 1, "MOTH", c_csir_x,c_csir_y,c_csir_z, 0, "ONLY" );

  // CsI Cylinder
  double  cscy_thick = 2.0; // 20 mm thick SUS
  double  cscy_len  = csir_len - 5;
  double  cscy_rmax = csir_rmax;
  double  cscy_rmin = cscy_rmax - cscy_thick;
  float  p_cscy[] = { cscy_rmin,cscy_rmax, cscy_len/2 };
  GSVOLU( "CSCY","TUBE",n_iron,p_cscy,sizeof(p_cscy)/sizeof(float),ivol );
  if ( DetUseCSIC[0] || DetUseCSEG[0] )
    GSPOS( "CSCY", 1, "CSI ", 0,0,2.5, 0, "ONLY" );

  // CsI Dummy Inner Compartment 1,2
  double  csh1_len = csir_len;
  double  csh1_xyo = (csic_xy+csic_gap*2)*13;
  double  csh1_xyi = (csic_xy+csic_gap*2)*5;
  float  p_csh1[] = { -360./4/2, 360, 4, 2,
                      -csh1_len/2, csh1_xyi/2, csh1_xyo/2,
                      +csh1_len/2, csh1_xyi/2, csh1_xyo/2 };
  GSVOLU( "CSH1","PGON",n_vacuum,p_csh1,sizeof(p_csh1)/sizeof(float),ivol );
  if ( DetUseCSIC[0] || DetUseCSEG[0] )
    GSPOS( "CSH1", 1, "CSI ", 0,0,0, 0, "ONLY" );

  double  csh2_len = csir_len;
  double  csh2_xyo = (csic_xy+csic_gap*2)*19;
  double  csh2_xyi = (csic_xy+csic_gap*2)*13;
  float  p_csh2[] = { -360./4/2, 360, 4, 2,
                      -csh2_len/2, csh2_xyi/2, csh2_xyo/2,
                      +csh2_len/2, csh2_xyi/2, csh2_xyo/2 };
  GSVOLU( "CSH2","PGON",n_vacuum,p_csh2,sizeof(p_csh2)/sizeof(float),ivol );
  if ( DetUseCSIC[0] || DetUseCSEG[0] )
    GSPOS( "CSH2", 1, "CSI ", 0,0,0, 0, "ONLY" );
  
  // Small Edge Calorimator
  //if ( DetUseCSEG[0] ) //////
  GSVOLU( "CSEG","TRAP",n_lead,0,0,ivol );
  double  csvt_bot[3],csvt_hi[3];
  // #a
  csvt_bot[0] = 4.1939;
  csvt_hi[0]  = 12.2741;
  // #b
  csvt_bot[1] = 6.2590;
  csvt_hi[1]  = 11.7946;
  // #c
  csvt_bot[2] = 5.5711;
  csvt_hi[2]  = 7.5529;

  // Locate CsI Crystal
  char  csiblock[][28] = {
    // 0         1         2      
    { ".........a3210123a........." },  //  0
    { ".......b40000000004b......." },  //  1
    { ".....c500000000000005c....." },  //  2
    { "....6000000000000000006...." },  //  3
    { "...600000000000000000006..." },  //  4
    { "..c000000000000000000000c.." },  //  5
    { "..50000000000000000000005.." },  //  6
    { ".b00000000000000000000000b." },  //  7
    { ".4000000000000000000000004." },  //  8
    { "a0000000000000000000000000a" },  //  9
    { "300000000000000000000000003" },  // 10
    { "20000000000.....00000000002" },  // 11
    { "10000000000.....00000000001" },  // 12
    { "00000000000.....00000000000" } };// 13
  
  int  id = 0;
  for( int y=0;y<27;y++ ){
    for( int x=0;x<27;x++ ){
      int  yy = (y<13) ? y : 26-y;
      double c_csic_x = (x-13)*(csic_xy+csic_gap*2);
      double c_csic_y = (y-13)*(csic_xy+csic_gap*2);
      double c_csic_z = -csir_len/2 + 5 + csic_len/2;
      if ( csiblock[yy][x] == '0' ){
        id++;
        if ( DEBUG )
          std::cout <<"CsI(" << id << ")=(" <<  x << "," << y 
                    << ") : " << csiblock[yy][x]
                    << std::endl;
        if ( DetUseCSIC[0] ){ //////
          if ( 13-6<=x && x<=13+6 && 13-6<=y && y<=13+6 )
            GSPOS( "CSIC",id,"CSH1",c_csic_x,c_csic_y,c_csic_z,0,"ONLY" );
          else if ( 13-9<=x && x<=13+9 && 13-9<=y && y<=13+9 )
            GSPOS( "CSIC",id,"CSH2",c_csic_x,c_csic_y,c_csic_z,0,"ONLY" );
          else
            GSPOS( "CSIC",id,"CSI ",c_csic_x,c_csic_y,c_csic_z,0,"ONLY" );
        }
      }
      else if ( '0' < csiblock[yy][x] && csiblock[yy][x] <= '9'  ){
        int  cd = csiblock[yy][x] - '0';
        id++;
        if ( DEBUG )
          std::cout <<"CsI(" << id << ")=(" <<  x << "," << y 
                    << ") : " << csiblock[yy][x]
                    << std::endl;
        if ( DetUseCSIC[0] ){ //////
          int  sec = (int)( fmod( Hep3Vector(x-13,y-13,0).phi()+2*M_PI,2*M_PI)
                            / (M_PI/4) );
          // 
          if ( !(cd == 2 || cd == 3) ){
            double  cbx = c_csic_x;
            double  cby = c_csic_y;
            double  cbz = c_csic_z;
            double  offset = csic_xy/2-csdb_y[cd]/2;
            int  rotb;
            if ( sec % 2 == 0 ){
              rotb = grot_z_32 + (24+sec*4) % 32;
              if ( sec == 0 ) cbx -= offset;
              if ( sec == 2 ) cby -= offset;
              if ( sec == 4 ) cbx += offset;
              if ( sec == 6 ) cby += offset;
            }
            else{
              rotb = grot_z_32 + (sec-1)*4;
              if ( sec == 1 ) cby -= offset;
              if ( sec == 3 ) cbx += offset;
              if ( sec == 5 ) cby += offset;
              if ( sec == 7 ) cbx -= offset;
            }
            float  p_csdb[] = { csic_xy/2,csdb_y[cd]/2,csic_len/2 };
            GSPOSP( "CSDB",id,"CSI ",cbx,cby,cbz,rotb,"ONLY",
                    p_csdb,sizeof(p_csdb)/sizeof(float) );
          }
          //
          double  ctx = c_csic_x;
          double  cty = c_csic_y;
          double  ctz = c_csic_z;
          double  offset   = csic_xy/2-(csdt_top[cd]+csdt_bot[cd])/4;
          double  offset_h = csic_xy/2-csdt_hi[cd]/2;
          int    rott;
          float  alpha = rad2deg( 
                         atan( (csdt_top[cd]-csdt_bot[cd])/2/csdt_hi[cd] ) );
          if ( cd == 2 || cd == 3 ){
            rott = ( sec%2 ) ? grot_z_32+(24+(sec-1)*4)%32 : grot_z_32+sec*4;
            alpha = ( sec%2 ) ? -alpha : alpha;
            if ( sec == 0 ) ctx -= offset;
            if ( sec == 1 ) cty -= offset;
            if ( sec == 2 ) cty -= offset;
            if ( sec == 3 ) ctx += offset;
            if ( sec == 4 ) ctx += offset;
            if ( sec == 5 ) cty += offset;
            if ( sec == 6 ) cty += offset;
            if ( sec == 7 ) ctx -= offset;
          }
          else {
            rott = ( sec%2 ) ?  grot_z_32+(sec-1)*4 : grot_z_32+(24+sec*4)%32;
            alpha = ( sec%2 ) ? alpha : -alpha;
            if ( sec == 0 ){ ctx += offset_h; cty -= offset;   }
            if ( sec == 1 ){ ctx -= offset;   cty += offset_h; }
            if ( sec == 2 ){ ctx += offset;   cty += offset_h; }
            if ( sec == 3 ){ ctx -= offset_h; cty -= offset;   }
            if ( sec == 4 ){ ctx -= offset_h; cty += offset;   }
            if ( sec == 5 ){ ctx += offset;   cty -= offset_h; }
            if ( sec == 6 ){ ctx -= offset;   cty -= offset_h; }
            if ( sec == 7 ){ ctx += offset_h; cty += offset;   }
          }
          float p_csdt[]={ csic_len/2, 0, 0,
                           csdt_hi[cd]/2,csdt_bot[cd]/2,csdt_top[cd]/2,alpha,
                           csdt_hi[cd]/2,csdt_bot[cd]/2,csdt_top[cd]/2,alpha };
          GSPOSP( "CSDT",id,"CSI ",ctx,cty,ctz,rott,"ONLY",
                  p_csdt,sizeof(p_csdt)/sizeof(float) );
        }
      }
      else if ( 'a' <= csiblock[yy][x] && csiblock[yy][x] <= 'z'  ){
        id++;
        int  cd = csiblock[yy][x] - 'a';
        int  sec = (int)( fmod( Hep3Vector(x-13,y-13,0).phi()+2*M_PI,2*M_PI)
                          / (M_PI/4) );
        double  ctx = c_csic_x;
        double  cty = c_csic_y;
        double  ctz = c_csic_z;
        double  offset   = csic_xy/2 - csvt_bot[cd]/4;
        double  offset_h = csic_xy/2 - csvt_hi[cd]/2;
        if ( sec == 0 ){ ctx -= offset;   cty -= offset_h; }
        if ( sec == 1 ){ ctx -= offset_h; cty -= offset;   }
        if ( sec == 2 ){ ctx += offset_h; cty -= offset;   }
        if ( sec == 3 ){ ctx += offset;   cty -= offset_h; }
        if ( sec == 4 ){ ctx += offset;   cty += offset_h; }
        if ( sec == 5 ){ ctx += offset_h; cty += offset;   }
        if ( sec == 6 ){ ctx -= offset_h; cty += offset;   }
        if ( sec == 7 ){ ctx -= offset;   cty += offset_h; }
        float  alpha = rad2deg( atan( csvt_bot[cd]/2/csvt_hi[cd] ) );
        alpha = ( sec%2 ) ? alpha : -alpha;
        int  rott = (sec%2) ? grot_z_32+(24+(sec-1)*4)%32 : grot_z_32+sec*4;
        float  p_csvt[]={ csic_len/2, 0, 0,
                          csvt_hi[cd]/2,csvt_bot[cd]/2, 0, alpha,
                          csvt_hi[cd]/2,csvt_bot[cd]/2, 0, alpha };
        if ( DetUseCSEG[0] ){ //////
          GSPOSP( "CSEG",id,"CSI ",ctx,cty,ctz,rott,"ONLY",
                  p_csvt,sizeof(p_csvt)/sizeof(float) );
        }
      }
    }
  }
  
}

