// -*- C++ -*-
//
// GsimGeom_FrontBarrel.cc
// Geometry definition function
//
// Author:  Mitsuhiro YAMAGA (yamaga@post.kek.jp)
// Created: Thu Jul 12 15:12:46 JST 2001
//
// $Id$
//
// Revision History
//
// $Log$
//

#include <iostream>
#include <string>
#include <stdlib.h>
#include <math.h>
#include "CLHEP/Vector/ThreeVector.h"
#include "ccfortran/G321Common.h"
#include "ccfortran/ccfortran.h"
#include "GsimGeneral.h"
#include "GsimGeom.h"


//////
#define  DEBUG  0


//////
extern int  DetUseFBAR[]; // GsimGeom.h


//////
void
GsimGeom_FrontBarrel()
{
  //
  int  ivol=0;

  // Front Barrel
  double  fbar_height = 41.2984;
  double  fbar_len    = 275;
  double  fbar_rmin   = 30.74;//31.7;
  double  fba1_height = 37.8304;
  double  fba1_bottom = 12.2291;
  double  fba1_top    = fba1_bottom + fba1_height*tan( M_PI/8 );
  double  fba1_alpha  = atan( (fba1_top-fba1_bottom)/2/fba1_height );
  double  fba1_alpdeg = rad2deg( fba1_alpha );
  float  p_fba1[] = { fbar_len/2, 0, 0, 
                      fba1_height/2,fba1_bottom/2,fba1_top/2,fba1_alpdeg,
                      fba1_height/2,fba1_bottom/2,fba1_top/2,fba1_alpdeg };
  GSVOLU( "FBA1","TRAP",n_lead,p_fba1,sizeof(p_fba1)/sizeof(float),ivol );
  for( int i=0;i<16;i++ ){
    double  x = fbar_rmin + fba1_height/2;
    double  y = fba1_bottom/2 -(fba1_top+fba1_bottom)/4;
    double  z = fbar_len/2;
    Hep3Vector c( x,y,z );
    c.rotateZ( 2*M_PI*i/16 );
    int  rotm = grot_z_32+(24+i*2)%32;
    if ( DetUseFBAR[0] ){ //////
      GSPOS( "FBA1",i+1,"MOTH",c.x(),c.y(),c.z(),rotm,"ONLY" );
    }
  }
  // 
  double  fba2_height = fbar_height - fba1_height;
  double  fba2_bottom = fba1_top;
  double  fba2_top    = 14.05;
  double  fba2_alpha  = atan( (fba2_top-fba2_bottom)/2/fba2_height );
  double  fba2_alpdeg = rad2deg( fba2_alpha );
  float  p_fba2[] = { fbar_len/2, 0, 0, 
                      fba2_height/2,fba2_bottom/2,fba2_top/2,fba2_alpdeg,
                      fba2_height/2,fba2_bottom/2,fba2_top/2,fba2_alpdeg };
  GSVOLU( "FBA2","TRAP",n_lead,p_fba2,sizeof(p_fba2)/sizeof(float),ivol );
  for( int i=0;i<16;i++ ){
    double  x = fbar_rmin + fbar_height - fba2_height/2;
    double  y = fba1_bottom/2 -(fba2_top+fba2_bottom)/4;
    double  z = fbar_len/2;
    Hep3Vector c( x,y,z );
    c.rotateZ( 2*M_PI*i/16 );
    int  rotm = grot_z_32+(24+i*2)%32;
    if ( DetUseFBAR[0] ){ //////
      GSPOS( "FBA2",i+1,"MOTH",c.x(),c.y(),c.z(),rotm,"ONLY" );
    }
  }
  // Front Barrel Backbone
  double  fbab_rmin = fbar_rmin + 41.3; // > fbar_rmin + fbar_height
  double  fbab_rmax = fbab_rmin + 2.2;
  float  p_fbab[] = { -360./16/2, 360, 16, 2,
                      -fbar_len/2, fbab_rmin, fbab_rmax,
                      +fbar_len/2, fbab_rmin, fbab_rmax };
  GSVOLU( "FBAB","PGON",n_iron,p_fbab,sizeof(p_fbab)/sizeof(float),ivol );
  double c_fbab_z = fbar_len/2;
  if ( DetUseFBAR[0] ){ //////
    GSPOS( "FBAB", 1, "MOTH", 0,0,c_fbab_z, 0, "ONLY" );
  }

}
