// -*- C++ -*-
//
// gustep.cc
// C++ interface of GUSTEP
//
// Author:  Mitsuhiro YAMAGA (yamaga@post.kek.jp)
// Created: Wed Jul 11 15:29:57 JST 2001
//
// $Id$
//
// Revision History
//
// $Log$
//

#include <iostream>
#include <string>
#include <math.h>
#include "ccfortran/G321Common.h"
#include "ccfortran/ccfortran.h"
#include "GsimGeneral.h"


//////
#define  DEBUG  0


//////
void
Gustep()
{
  if ( DEBUG || gcflag_.ISWIT[3-1] > 0 )
    std::cerr << "Gustep" << std::endl;
  
  // Check Volume
  int  stop = GsimHits();

  if ( stop ){
    // Stop swimming
    gctrak_.ISTOP = stop;
  }
  else{
    // Store secondary particle
    for( int i=0;i<gcking_.NGKINE;i++ ){
      int  type = (int)gcking_.GKIN[i][4];
      if ( type == 4 ) // skip neutrino
        gcking_.IFLGK[i] = -1;
      else
        gcking_.IFLGK[i] = 1;
      GSKING( i+1 );
      // mechanism
      if ( 0 < gcking_.IFLGK[i] ){
        if ( gcking_.IFLGK[i] < MAX_MECH ){
          GenMechanism[gcking_.IFLGK[i]] = gcking_.KCASE;
        }
        else{
          static bool  first = true; 
          if ( first )
            std::cerr << "Gustep : #particle exceeds the size of mech[]\n"
                      << std::endl;
          first = false;
        }
      }
    }
  }
  if ( DEBUG || gcflag_.ISWIT[3-1] > 1 ){
    std::cerr << "Gustep : NGKINE=" << gcking_.NGKINE << std::endl;
  }

  //
  if ( Display ){ 
    if ( gcflag_.ISWIT[4-1] == 1 )
      GSXYZ();
    else if ( gcflag_.ISWIT[4-1] == 3 )
      GDCXYZ();
  }

}

/// FORTRAN/C interface
extern"C"{
  void  gustep_(){ Gustep(); }
}

