// -*- C++ -*-
//
// TMNtuple.cc
// NTUPLE accessing class
//
// Author:  Mitsuhiro YAMAGA (yamaga@post.kek.jp)
// Created: Tue Sep 11 17:17:42 JST 2001
//
// $Id$
//
// Revision History
//
// $Log$
//

#include <iostream>
#include <string>
#include <string.h>
#include "ccfortran/ccfortran.h"
#include "TupleManager/TupleManager.h"
#include "TupleManager/TMNtuple.h"


//
extern int  quest_[100]; // common/quest/iquest(100)


//////// TMNtuple
//
TMNtuple::TMNtuple( const TupleManager* mn )
  : m_bufinit( false ),
    m_type( 0 ),
    m_manager( mn ),
    m_title( "" ),
    m_block( "" ),
    m_cname( "" ),
    m_hid( 0 ),
    m_vars( 0 )
{

}


//
TMNtuple::~TMNtuple()
{
  
}


// CWN
void
TMNtuple::book( int hid, const std::string& title,
                const std::string& block,
                void* vars, const std::string& cname )
{
  m_type  = TM_CWN;
  m_hid   = hid;
  m_title = title;
  m_block = block;
  m_vars  = (float*)vars;
  m_cname = cname;

  if ( m_manager->status() == TM_Write ){
    int  err = 0;
    HBSET( "BSIZE",8192,err );
    HCDIR( m_manager->cdir(), " " );
    HBNT( m_hid, m_title, "D" );
    HBNAME( m_hid, m_block, m_vars, m_cname );
  }
  else if ( m_manager->status() == TM_Read ){
    HRIN( m_hid,999,0 ); // dummy read     
  }
  
}


// CWN, read only
void
TMNtuple::book( int hid, const std::string& block, void* vars )
{
  m_type  = TM_CWN;
  m_hid   = hid;
  m_title = "";
  m_block = block;
  m_vars  = (float*)vars;
  m_cname = "";

  HRIN( m_hid,999,0 ); // dummy read 

  if ( m_manager->status() == TM_Write ){
    std::cout << "TMNtuple::book : Data name and type not defined."
              << std::endl;
  }
  
}


// RWN
void
TMNtuple::book( int hid, const std::string& title,
                void* vars, const std::string& varname )
{
  m_type  = TM_RWN;
  m_hid   = hid;
  m_title = title;
  m_block = "";
  m_vars  = (float*)vars;
  m_cname = "";

  // varname
  char*  svar = new char[varname.length()+1];
  strcpy( svar,varname.c_str() );
  char* cp   = svar;
  //char* next = cp;
  //cp = strsep( &next," " );
  cp = strtok( cp," " );
  int  nvar = 0;
  do{
    m_cname += std::string( cp + std::string( "        " ),0,8 );
    nvar++;
    //cp = strsep( &next," " );
    cp = strtok( NULL," " );
  } while( cp != NULL );

  // book
  HBOOKN( hid,title,nvar,m_manager->dir(),81920,m_cname );

  delete svar;
}


// RWN, read only
void
TMNtuple::book( int hid, void* vars )
{
  m_type  = TM_RWN;
  m_hid   = hid;
  m_title = "";
  m_block = "";
  m_vars  = (float*)vars;
  m_cname = "";
}


//
int
TMNtuple::dump()
{
  if ( m_type == TM_CWN ){// CWN
    HFNT( m_hid );
    //HFNTB( m_hid,m_block );
  }
  else if ( m_type == TM_RWN ){ //RWN
    HFN( m_hid,m_vars );
  }
  
  return( quest_[0] );
}


//
int
TMNtuple::read( int eventNo )
{
  int  ierr;
  
  if ( m_type == TM_CWN ){
    if ( !m_bufinit ){
      HCDIR( m_manager->cdir(), " " );
      HBNAME( m_hid, " ", 0 ,"$CLEAR" );
      HBNAME( m_hid, m_block, m_vars, "$SET" );
      HGNT( m_hid, eventNo, ierr );
      m_bufinit = true;
    }
    else{
      HGNTF( m_hid, eventNo, ierr );
    }
  }
  else if ( m_type == TM_RWN ){
    if ( !m_bufinit ){
      HGNPAR( m_hid,"TMNtuple::read" );
      m_bufinit = true;
    }
    HGNF( m_hid, eventNo, m_vars, ierr );
  }

  return( ierr );
}


//
int
TMNtuple::totalEntries()
{
  int  ent;

  HCDIR( m_manager->cdir(), " " );
  HNOENT( m_hid, ent );

  return( ent );
}
