// -*- C++ -*-
//
// TupleManager.cc
// HBOOK/NTUPLE accessing classes
//
// Author:  Mitsuhiro YAMAGA (yamaga@post.kek.jp)
// Created: Tue Sep 11 17:17:42 JST 2001
//
// $Id$
//
// Revision History
//
// $Log$
//

#include <iostream>
#include "ccfortran/ccfortran.h"
#include "TupleManager/TupleManager.h"
#include "TupleManager/TMNtuple.h"
#include "TupleManager/TMHistogram.h"

//
extern int  quest_[100]; // common/quest/iquest(100)


//////// TupleManager
//
TupleManager::TupleManager()
  : m_fname( "" ),
    m_dir( "" ),
    m_cdir( "//" ),
    m_lun( 0 ),
    m_status( 0 )
{
  m_ntlist.clear();
  m_hilist.clear();

}


//
TupleManager::~TupleManager()
{
  if ( m_lun )
    this->close();

}


//
void
TupleManager::init( int lim )
{
  static bool  init = false;
  if ( !init ){
    //std::cerr << "HLIMIT=" << lim << std::endl;
    HLIMIT( lim );
    init = true;
  }
}


//
int
TupleManager::ropen( const std::string& fname, const std::string& dir,
                     int lun )
{
  if ( m_lun )
    this->close();

  m_fname  = fname;
  m_dir    = dir;
  m_cdir   = "//" + dir;
  m_lun    = lun;
  m_status = TM_Read;

  int  stat;
  HROPEN( lun,dir,fname," CX",0,stat );
  std::cout << "TupleManager::ropen : " << fname << std::endl;

  return( stat );
}


int
TupleManager::wopen( const std::string& fname, const std::string& dir,
                     int lun )
{
  if ( m_lun )
    this->close();

  m_fname  = fname;
  m_dir    = dir;
  m_cdir   = "//" + dir;
  m_lun    = lun;
  m_status = TM_Write;

  int  stat;
  HROPEN( m_lun,m_dir,m_fname,"NCX",1024,stat );
  std::cout << "TupleManager::wopen : " << fname << std::endl;

  return( stat );
}


//
int
TupleManager::close()
{
  if ( m_lun ){
    if ( m_status == TM_Write )
      this->write( 0 ); // dump all the histograms
    HREND( m_dir );
    m_lun = 0;
    m_status = 0;

    for( std::list<TMNtuple*>::iterator i = m_ntlist.begin();
         i != m_ntlist.end(); i++ )
      delete *i;
    m_ntlist.clear();
    for( std::list<TMHistogram*>::iterator i = m_hilist.begin();
         i != m_hilist.end(); i++ )
      delete *i;
    m_hilist.clear();
  }
  return( quest_[0] );
}


//
int
TupleManager::write( int hid )
{
  int  cycle;
  
  HCDIR( m_cdir, " " );
  HROUT( hid, cycle, " " );

  return( quest_[0] );
}


// CWN
TMNtuple*
TupleManager::ntuple( int hid, const std::string& title,
                      const std::string& block, 
                      void* vars, const std::string& cname )
{
  TMNtuple* nt = new TMNtuple( this );
  nt->book( hid,title,block,vars,cname );
  m_ntlist.push_back( nt );
  
  return( nt );
}


// CWN (read only)
TMNtuple*
TupleManager::ntuple( int hid, const std::string& block, void* vars )
{
  TMNtuple* nt = new TMNtuple( this );
  nt->book( hid,block,vars );
  m_ntlist.push_back( nt );

  return( nt );
}


// RWN
TMNtuple*
TupleManager::ntupleR( int hid, const std::string& title,
                       void* vars, const std::string& varname )
{
  TMNtuple* nt = new TMNtuple( this );
  nt->book( hid,title,vars,varname );
  m_ntlist.push_back( nt );

  return( nt );
}


// RWN (read only)
TMNtuple*
TupleManager::ntupleR( int hid, void* vars )
{
  TMNtuple* nt = new TMNtuple( this );
  nt->book( hid,vars );
  m_ntlist.push_back( nt );

  return( nt );
}


//
TMHistogram*
TupleManager::histogram( int hid, const std::string& title,
                         int xbin,double xmin, double xmax )
{
  TMHistogram* hi = new TMHistogram( this );
  hi->book( hid,title,xbin,xmin,xmax );
  m_hilist.push_back( hi );

  return( hi );
}


//
TMHistogram*
TupleManager::histogram( int hid, const std::string& title,
                         int xbin,double xmin, double xmax,
                         int ybin,double ymin, double ymax )
{
  TMHistogram* hi = new TMHistogram( this );
  hi->book( hid,title,xbin,xmin,xmax,ybin,ymin,ymax );
  m_hilist.push_back( hi );

  return( hi );
}

